/* Gdf (Graphically represents file system usage).                            */
/* Copyright (C) 1999 Denis NICOLAS                                           */

/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */

/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              */
/* GNU General Public License for more details.                               */

/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software Foundation,    */
/* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            */

/* Contact: denis.nicolas@iname.com                                           */


#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <string.h>


GtkWidget *label_type;
GtkWidget *label_size;
GtkWidget *label_avail;
GtkWidget *label_capacity;
GtkWidget *bar;
GtkWidget *optionmenu;

char *prog;
char *fullName;


void scan (GtkMenuItem *MENU_ITEM, gpointer *data) {
  
  char *token;
  char *line;
  char *l;
  char *fs;
  char *chosen_line;
  const int maxline = 1024;
  int i;
  int chosen_i;
  float value;
  GtkWidget *menuitem;
  GtkWidget *optionmenu_menu;

  if (freopen (fullName, "w+", stdout) == NULL) {
    fprintf (stderr, "%s: couldn't open \"%s\".\n", prog, fullName);
    exit (1);
  }
  
  system ("df -h -T");
  rewind (stdout);

  while (getc (stdout) != '\n') ;

  optionmenu_menu = gtk_menu_new ();
  line = (char *) malloc (maxline);
  l = (char *) malloc (maxline);
  chosen_line = (char *) malloc (maxline);
  i = 0;
  chosen_i = 0;
  while (fgets (line, maxline, stdout)) {
    strcpy (l, line);
    if (i == 0) {
      strcpy (chosen_line, line);
    }
    token = strtok (l, " ");
    strtok (NULL, "%");
    fs = (char *) malloc (maxline);
    sprintf(fs, "%s (%s)", strtok (NULL, " \n"), token);

    menuitem = gtk_menu_item_new_with_label (fs);
    gtk_widget_show (menuitem);
    gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
    gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
			GTK_SIGNAL_FUNC (scan), (gpointer) fs);

    if (data && (strcmp(fs, (char *) data) == 0)) {
      strcpy (chosen_line, line);
      chosen_i = i;
    }

    i++;
  }
  free (line);
  free (l);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu), optionmenu_menu);
  gtk_option_menu_set_history (GTK_OPTION_MENU (optionmenu), chosen_i);
  
  strtok (chosen_line, " ");
  free (chosen_line);
  token = strtok (NULL, " ");
  gtk_label_set (GTK_LABEL (label_type), token);

  token = strtok (NULL, " ");
  gtk_label_set (GTK_LABEL (label_size), token);
  
  strtok (NULL, " ");
  token = strtok (NULL, " ");
  gtk_label_set (GTK_LABEL (label_avail), token);

  token = strtok (NULL, " ");
  gtk_label_set (GTK_LABEL (label_capacity), token);
  value = atoi (strtok (token, "%")) / 100.0;
  gtk_progress_bar_update (GTK_PROGRESS_BAR (bar), value);

}


gint Quit (GtkWidget *Widget, gpointer *Data) {
  
  gtk_main_quit ();
  return (TRUE);
  
}


void notice (void) {

  printf ("\nGdf version 0.4, Copyright (C) 1999 Denis NICOLAS\n");
  printf ("Gdf comes with ABSOLUTELY NO WARRANTY.\n");
  printf ("This is free software, and you are welcome\n");
  printf ("to redistribute it under certain conditions.\n");
  printf ("See the GNU General Public License for details.\n");

}


int main (int argc, char *argv[]) {

  GtkWidget *window;
  GtkWidget *main_box;
  GtkWidget *box;
  GtkWidget *align;

  GtkWidget *label_T;
  GtkWidget *label_S;
  GtkWidget *label_A;
  GtkWidget *label_C;

  GtkTooltips *tip;
  GdkColor bg = { 65535, 65535, 65535, 0 };
  GdkColor fg = { 0, 0, 0, 0 };

  const char *path = getenv ("HOME");
  static const char *df_out_filename = ".df_output.txt";

  notice ();

  prog = argv[0];
  fullName = (char *) malloc (strlen (path) + strlen (df_out_filename) + 1);
  // + 1 pour le '/'

  sprintf (fullName, "%s/%s", path, df_out_filename);
  
  gtk_init (&argc, &argv);
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), "Gdf");
  main_box = gtk_vbox_new (FALSE, 5);
  gtk_container_add (GTK_CONTAINER (window), main_box);
  gtk_container_border_width (GTK_CONTAINER (window), 5);
  gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		     GTK_SIGNAL_FUNC (Quit), NULL);

  align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
  gtk_widget_show (align);
  gtk_box_pack_start (GTK_BOX (main_box), align, TRUE, TRUE, 5);
  
  optionmenu = gtk_option_menu_new ();
  gtk_widget_show (optionmenu);
  gtk_container_add (GTK_CONTAINER (align), optionmenu);
  tip = gtk_tooltips_new ();
  gtk_tooltips_set_tip (tip, optionmenu, "Mount point (device)", NULL);
  gtk_tooltips_set_colors (tip, &bg, &fg);

  box = gtk_hbox_new (TRUE, 5);

  label_T = gtk_label_new ("Filesystem type:");
  align = gtk_alignment_new (0.0, 0.5, 0.0, 1.0);
  gtk_container_add (GTK_CONTAINER (align), label_T);
  gtk_widget_show (label_T);
  gtk_box_pack_start (GTK_BOX (box), align, TRUE, TRUE, 5);
  gtk_widget_show (align);
  

  label_type = gtk_label_new ("");
  align = gtk_alignment_new (1.0, 0.5, 0.0, 1.0);
  gtk_container_add (GTK_CONTAINER (align), label_type);
  gtk_widget_show (label_type);
  gtk_box_pack_start (GTK_BOX (box), align, TRUE, TRUE, 5);
  gtk_widget_show (align);

  gtk_container_add (GTK_CONTAINER (main_box), box);
  gtk_widget_show (box);

  box = gtk_hbox_new (TRUE, 5);

  label_S = gtk_label_new ("Total size:");
  align = gtk_alignment_new (0.0, 0.5, 0.0, 1.0);
  gtk_container_add (GTK_CONTAINER (align), label_S);
  gtk_widget_show (label_S);
  gtk_box_pack_start (GTK_BOX (box), align, TRUE, TRUE, 5);
  gtk_widget_show (align);


  label_size = gtk_label_new ("");
  align = gtk_alignment_new (1.0, 0.5, 0.0, 1.0);
  gtk_container_add (GTK_CONTAINER (align), label_size);
  gtk_widget_show (label_size);
  gtk_box_pack_start (GTK_BOX (box), align, TRUE, TRUE, 5);
  gtk_widget_show (align);

  gtk_container_add (GTK_CONTAINER (main_box), box);
  gtk_widget_show (box);

  box = gtk_hbox_new (TRUE, 5);

  label_A = gtk_label_new ("Space available:");
  align = gtk_alignment_new (0.0, 0.5, 0.0, 1.0);
  gtk_container_add (GTK_CONTAINER (align), label_A);
  gtk_widget_show (label_A);
  gtk_box_pack_start (GTK_BOX (box), align, TRUE, TRUE, 5);
  gtk_widget_show (align);

  label_avail = gtk_label_new ("");
  align = gtk_alignment_new (1.0, 0.5, 0.0, 1.0);
  gtk_container_add (GTK_CONTAINER (align), label_avail);
  gtk_widget_show (label_avail);
  gtk_box_pack_start (GTK_BOX (box), align, TRUE, TRUE, 5);
  gtk_widget_show (align);

  gtk_container_add (GTK_CONTAINER (main_box), box);
  gtk_widget_show (box);

  box = gtk_hbox_new (TRUE, 5);

  label_C = gtk_label_new ("Space used:");
  align = gtk_alignment_new (0.0, 0.5, 0.0, 1.0);
  gtk_container_add (GTK_CONTAINER (align), label_C);
  gtk_widget_show (label_C);
  gtk_box_pack_start (GTK_BOX (box), align, TRUE, TRUE, 5);
  gtk_widget_show (align);

  label_capacity = gtk_label_new ("");
  align = gtk_alignment_new (1.0, 0.5, 0.0, 1.0);
  gtk_container_add (GTK_CONTAINER (align), label_capacity);
  gtk_widget_show (label_capacity);
  gtk_box_pack_start (GTK_BOX (box), align, TRUE, TRUE, 5);
  gtk_widget_show (align);

  gtk_container_add (GTK_CONTAINER (main_box), box);
  gtk_widget_show (box);

  bar = gtk_progress_bar_new ();
  gtk_box_pack_start (GTK_BOX (main_box), bar, TRUE, TRUE, 5);
  gtk_widget_show (bar);

  scan (NULL, NULL);

  gtk_widget_show (main_box);
  gtk_widget_show (window);

  gtk_main (); 
  remove (fullName);
  
  exit (0);

}
